import traceback

from pycrescolib.admin import admin
from pycrescolib.agents import agents
from pycrescolib.dataplane import dataplane
from pycrescolib.globalcontroller import globalcontroller
from pycrescolib.logstreamer import logstreamer
from pycrescolib.messaging import messaging
from pycrescolib.wc_interface import ws_interface
import ssl

try:
    _create_unverified_https_context = ssl._create_unverified_context
except AttributeError:
    # Legacy Python that doesn't verify HTTPS certificates by default
    pass
else:
    # Handle target environment that doesn't support HTTPS verification
    ssl._create_default_https_context = _create_unverified_https_context

class clientlib(object):

    def __init__(self,host, port, service_key):
        self.host = host
        self.port = port
        self.service_key = service_key
        self.ws_interface = ws_interface()
        self.messaging = messaging(self.ws_interface)
        self.agents = agents(self.messaging)
        self.admin = admin(self.messaging)
        self.globalcontroller = globalcontroller(self.messaging)

    def connect(self):
        try:
            ws_url = 'wss://' + self.host + ':' + str(self.port) +'/api/apisocket'
            isWSConnected = self.ws_interface.connect(ws_url)
            return isWSConnected
        except:
            traceback.print_exc()
            return False

    def connected(self):
        try:
            return self.ws_interface.connected()
        except:
            return False

    def close(self):
        if self.ws_interface.connected():
            self.ws_interface.close()

    def get_dataplane(self, stream_name, callback=None):
        return dataplane(self.host, self.port, stream_name, callback)

    def get_logstreamer(self, callback=None):
        return logstreamer(self.host, self.port, callback)







import json

from pycrescolib.utils import decompress_param, get_jar_info, compress_param, encode_data


class globalcontroller(object):

    def __init__(self, messaging):
        self.messaging = messaging

    def submit_pipeline(self, cadl):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'gpipelinesubmit'
        message_payload['action_gpipeline'] = compress_param(json.dumps(cadl))
        message_payload['action_tenantid'] = '0'

        retry = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)
        # returns status and gpipeline_id
        return retry

    def remove_pipeline(self, pipeline_id):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'gpipelineremove'
        message_payload['action_pipelineid'] = pipeline_id
        retry = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)

        return retry

    def get_pipeline_list(self):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'getgpipelinestatus'

        reply = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)
        reply = json.loads(decompress_param(reply['pipelineinfo']))['pipelines']

        return reply

    def get_pipeline_info(self, pipeline_id):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'getgpipeline'
        message_payload['action_pipelineid'] = pipeline_id

        reply = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)
        reply = json.loads(decompress_param(reply['gpipeline']))

        return reply

    def get_pipeline_status(self, pipeline_id):

        reply = self.get_pipeline_info(pipeline_id)
        status_code = int(reply['status_code'])
        return status_code

    def get_agent_list(self, dst_region=None):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'listagents'
        if dst_region is not None:
            message_payload['action_region'] = dst_region

        reply = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)

        reply = json.loads(decompress_param(reply['agentslist']))['agents']

        '''
        for agent in reply:
            dst_agent = agent['name']
            dst_region = agent['region']
            r = self.get_agent_resources(dst_region,dst_agent)
            print(r)
        '''

        return reply

    def get_agent_resources(self, dst_region, dst_agent):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'resourceinfo'
        message_payload['action_region'] = dst_region
        message_payload['action_agent'] = dst_agent

        reply = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)

        reply = json.loads(json.loads(decompress_param(reply['resourceinfo']))['agentresourceinfo'][0]['perf'])

        return reply

    def get_plugin_list(self):
        # this code makes use of a global message to find a specific plugin type, then send a message to that plugin
        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'listplugins'

        result = self.messaging.global_controller_msgevent(message_event_type, message_payload)

        pluginslist = json.loads(decompress_param(result['pluginslist']))

        plugin_name = 'io.cresco.repo'
        pluginlist = pluginslist['plugins']
        for plugin in pluginlist:
            if plugin['pluginname'] == plugin_name:
                break;

        message_payload['action'] = 'repolist'
        for i in range(10):
            result = self.messaging.global_plugin_msgevent(True, message_event_type, message_payload, plugin['region'], plugin['agent'], plugin['name'])
            print(result)

    def upload_plugin_global(self, jar_file_path):

        #get data from jar
        configparams = get_jar_info(jar_file_path)

        # "configparams"
        '''
        configparams = dict()
        configparams['pluginname'] = 'io.cresco.cepdemo'
        configparams['version'] = '1.0.0.SNAPSHOT-2020-09-01T203900Z'
        configparams['md5'] = '34de550afdac3bcabbbac99ea5a1519c'
        '''

        #read input file
        in_file = open(jar_file_path, "rb")  # opening for [r]eading as [b]inary
        jar_data = in_file.read()  # if you only wanted to read 512 bytes, do .read(512)
        in_file.close()

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'savetorepo'
        message_payload['configparams'] = compress_param(json.dumps(configparams))
        message_payload['jardata'] = encode_data(jar_data)

        reply = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)

        # returns reply with status and pluginid
        return reply

    def get_region_resources(self, dst_region):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'resourceinfo'
        message_payload['action_region'] = dst_region

        reply = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)

        #reply = json.loads(json.loads(decompress_param(reply['resourceinfo']))['agentresourceinfo'][0]['perf'])
        reply = json.loads(decompress_param(reply['resourceinfo']))

        return reply

    def get_region_list(self):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'listregions'

        reply = self.messaging.global_controller_msgevent(True, message_event_type, message_payload)
        reply = json.loads(decompress_param(reply['regionslist']))['regions']

        return reply


import gzip
import io
import base64
from zipfile import ZipFile
import hashlib

def compress_param(params):
    out = io.BytesIO()

    with gzip.GzipFile(fileobj=out, mode='w') as fo:
        fo.write(params.encode())

    bytes_obj = out.getvalue()
    return base64.b64encode(bytes_obj).decode('utf-8')

def compress_data(byte_data):
    out = io.BytesIO()

    with gzip.GzipFile(fileobj=out, mode='w') as fo:
        fo.write(byte_data)

    bytes_obj = out.getvalue()
    return base64.b64encode(bytes_obj).decode('utf-8')

def encode_data(byte_data):

    return base64.b64encode(byte_data).decode('utf-8')


def decompress_param(param):
    compressed_bytes = base64.b64decode(param)
    uncompressed_bytes = gzip.decompress(compressed_bytes).decode()
    return uncompressed_bytes

def get_jar_info(jar_file_path):

    params = dict()

    with ZipFile(jar_file_path, 'r') as myzip:
        myfile = myzip.read(name='META-INF/MANIFEST.MF')
        for line in myfile.decode().split('\n'):
            line = line.strip().split(': ')
            if line[0] == 'Bundle-SymbolicName':
                params['pluginname'] = line[1]
            if line[0] == 'Bundle-Version':
                params['version'] = line[1]

    params['md5'] = hashlib.md5(open(jar_file_path, 'rb').read()).hexdigest()

    return params


import ssl
import time
import json
import websocket
try:
    import thread
except ImportError:
    import _thread as thread

class logstreamer(object):

    def __init__(self, host, port, callback):
        self.host = host
        self.port = port
        self.ws = None
        self.isActive = False
        self.message_count = 0
        self.callback = callback

    def on_message(self, ws, message):

        if (self.message_count == 0):
            json_incoming = json.loads(message)
            if int(json_incoming['status_code']) == 10:
                self.isActive = True
        else:
            if self.callback is not None:
                self.callback(message)
            else:
                print("Log Message = " + str(message))

        self.message_count += 1

    def update_config(self, dst_region, dst_agent):

        '''
        String region_id = sst[0];
        String agent_id = sst[1];
        String baseclass = sst[2];
        String loglevel = sst[3];

        dst_region = "global-region"
        dst_agent = "global-controller"

        '''
        #message = 'global-region,global-controller,io.cresco,Trace'
        #message = dst_region + ',' + dst_agent + ',Info,default'
        message = dst_region + ',' + dst_agent + ',Trace,default'

        self.ws.send(message)

    def on_error(self, ws, error):
        print(error)

    def on_close(self, ws):
        print("### closed logstreamer ###")

    def on_open(self, ws):
        #self.ws.send(self.stream_name)

        '''
        def run(*args):
            self.ws.send(self.stream_name)
            for i in range(30):
                time.sleep(1)
                # ws.send("Hello %d" % i)
                # ws.send("cat")
            time.sleep(1)
            self.ws.close()
            print("thread terminating...")

        thread.start_new_thread(run, ())
        '''

        #def run(*args):
        #    self.ws.send(self.stream_name)

        #thread.start_new_thread(run, ())


    def close(self):
        self.ws.close()

    def connect(self):

        def run(*args):
            ws_url = 'wss://' + self.host + ':' + str(self.port) + '/api/logstreamer'
            websocket.enableTrace(False)
            self.ws = websocket.WebSocketApp(ws_url,
                                             on_message=self.on_message,
                                             on_error=self.on_error,
                                             on_close=self.on_close,
                                             #sslopt = {"cert_reqs": ssl.CERT_NONE},
                                             header={'cresco_service_key': 'abc-8675309'}
                                             )
            self.ws.on_open = self.on_open
            self.ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})

        thread.start_new_thread(run, ())

        while not self.isActive:
            time.sleep(1)

        '''
        ws_url = 'ws://' + self.host + ':' + str(self.port) +'/api/dataplane'
        websocket.enableTrace(True)
        self.ws = websocket.WebSocketApp(ws_url,
                                    on_message=self.on_message,
                                    on_error=self.on_error,
                                    on_close=self.on_close)
        self.ws.on_open = self.on_open
        self.ws.run_forever()
        '''


import ssl

from websocket import create_connection

class ws_interface(object):

    def __init__(self):
        self.url = None
        self.ws = None

    def connect(self, url):
        self.url = url
        self.ws = create_connection(self.url, sslopt={"cert_reqs": ssl.CERT_NONE}, header={'cresco_service_key': 'abc-8675309'})
        return self.ws.connected

    def connected(self):
        if self.ws is None:
            return False
        else:
            return self.ws.connected

    def close(self):
        if self.ws is not None:
            self.ws.close()



import json
import ssl
import time
import websocket

try:
    import thread
except ImportError:
    import _thread as thread

class dataplane(object):

    def __init__(self, host, port, stream_name, callback):
        self.host = host
        self.port = port
        self.stream_name = stream_name
        self.ws = None
        self.isActive = False
        self.message_count = 0
        self.callback = callback

    def is_active(self):
        return self.isActive

    def on_data(self, ws, incoming_string, data_type, continue_flag):
        print('on_data message: incoming_string' + str(incoming_string) + ' data_type:' + str(data_type) + ' continue_flag: ' + str(continue_flag))

    def on_message(self, ws, message):

        if not self.isActive:
            try:
                json_incoming = json.loads(message)
                if int(json_incoming['status_code']) == 10:
                    self.isActive = True
            except:
                print('DP Activation failure: ' + str(message))

        else:
            if self.callback is not None:
                self.callback(message)
            else:
                print("DP Message = " + str(message))
                print("DP Message Type = " + str(type(message)))

        self.message_count += 1

    def on_error(self, ws, error):
        print(error)

    def on_close(self, ws, close_code, close_desc):

        print('closed dp_id: ' + str(self.dp_id))

        if (close_code is not None) and (close_desc is not None):
            print('dp closed: code:' + str(close_code) + ' desc: ' + str(close_desc))

    def on_open(self, ws):
        self.ws.send(self.stream_name)

    def close(self):
        self.ws.close()

    def send(self, data):
        if(self.isActive):
            self.ws.send(data)
        else:
            print('send(): not sending, not active')

    def connect(self):

        def run(*args):
            ws_url = 'wss://' + self.host + ':' + str(self.port) + '/api/dataplane'
            websocket.enableTrace(False)
            self.ws = websocket.WebSocketApp(ws_url,
                                             on_message=self.on_message,
                                             #on_data=self.on_data,
                                             on_error=self.on_error,
                                             on_close=self.on_close,
                                             #sslopt={"cert_reqs": ssl.CERT_NONE},
                                             header={'cresco_service_key': 'abc-8675309'}
                                             )
            self.ws.on_open = self.on_open
            self.ws.run_forever(sslopt={"cert_reqs": ssl.CERT_NONE})

        thread.start_new_thread(run, ())

        while not self.isActive:
            time.sleep(1)





class admin(object):

    def __init__(self, messaging):
        self.messaging = messaging


    def stopcontroller(self, dst_region, dst_agent):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'stopcontroller'


        self.messaging.global_agent_msgevent(False, message_event_type, message_payload, dst_region, dst_agent)

    def restartcontroller(self, dst_region, dst_agent):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'restartcontroller'


        self.messaging.global_agent_msgevent(False, message_event_type, message_payload, dst_region, dst_agent)


    def restartframework(self, dst_region, dst_agent):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'restartframework'


        self.messaging.global_agent_msgevent(False, message_event_type, message_payload, dst_region, dst_agent)

    def killjvm(self, dst_region, dst_agent):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'killjvm'


        self.messaging.global_agent_msgevent(False, message_event_type, message_payload, dst_region, dst_agent)



from setuptools import setup

setup(
    name='pycrescolib',
    version='0.12',
    packages=['pycrescolib'],
    url='http://cresco.io',
    license='Apache 2.0',
    author='Cresco Team',
    author_email='info@cresco.io',
    description='Python Cresco Client Library',
    install_requires=['websocket-client'],

)



import json

from pycrescolib.utils import compress_param, decompress_param, get_jar_info, encode_data


class agents(object):

    def __init__(self, messaging):
        self.messaging = messaging

    def is_controller_active(self, dst_region, dst_agent):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'iscontrolleractive'

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)
        reply = bool(reply['is_controller_active'])

        return reply

    def get_controller_status(self, dst_region, dst_agent):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'getcontrollerstatus'

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)
        reply = reply['controller_status']

        return reply

    def add_plugin_agent(self, dst_region, dst_agent, configparams, edges):

        # "configparams"
        '''
        configparams = dict()
        configparams['inode_id'] = 'api-' + str(uuid.uuid1())
        configparams['pluginname'] = 'io.cresco.cepdemo'
        configparams['version'] = '1.0.0.SNAPSHOT-2020-09-01T203900Z'
        configparams['md5'] = '34de550afdac3bcabbbac99ea5a1519c'
        configparams['jarfile'] = configparams['md5']
        configparams['mode'] = '0'
        '''

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'pluginadd'
        message_payload['configparams'] = compress_param(json.dumps(configparams))

        if edges is not None:
            message_payload['edges'] = compress_param(json.dumps(edges))

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)

        # returns reply with status and pluginid
        return reply

    def remove_plugin_agent(self, dst_region, dst_agent, plugin_id):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'pluginremove'
        message_payload['pluginid'] = plugin_id
        # dst_region = 'global-region'
        # dst_agent = 'global-controller'

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)
        # return reply with status code and pluginid
        return reply

    def list_plugin_agent(self, dst_region, dst_agent):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'pluginlist'

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)
        reply = json.loads(decompress_param(reply['plugin_list']))
        return reply

    def status_plugin_agent(self, dst_region, dst_agent, plugin_id):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'pluginstatus'
        message_payload['pluginid'] = plugin_id

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)
        #print(reply)
        #reply = json.loads(decompress_param(reply['plugin_status']))
        return reply

    def get_agent_info(self, dst_region, dst_agent):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'getagentinfo'

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)
        reply = reply['agent-data']
        return reply

    def get_agent_log(self, dst_region, dst_agent):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'getlog'

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)
        #reply = reply['agent-data']
        return reply


    def repo_pull_plugin_agent(self, dst_region, dst_agent, jar_file_path):

        #get data from jar
        configparams = get_jar_info(jar_file_path)

        # "configparams"
        '''
        configparams = dict()
        configparams['pluginname'] = 'io.cresco.cepdemo'
        configparams['version'] = '1.0.0.SNAPSHOT-2020-09-01T203900Z'
        configparams['md5'] = '34de550afdac3bcabbbac99ea5a1519c'
        '''

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'pluginrepopull'
        message_payload['configparams'] = compress_param(json.dumps(configparams))


        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)

        # returns reply with status and pluginid
        return reply

    def upload_plugin_agent(self, dst_region, dst_agent, jar_file_path):

        #get data from jar
        configparams = get_jar_info(jar_file_path)

        # "configparams"
        '''
        configparams = dict()
        configparams['pluginname'] = 'io.cresco.cepdemo'
        configparams['version'] = '1.0.0.SNAPSHOT-2020-09-01T203900Z'
        configparams['md5'] = '34de550afdac3bcabbbac99ea5a1519c'
        '''

        #read input file
        in_file = open(jar_file_path, "rb")  # opening for [r]eading as [b]inary
        jar_data = in_file.read()  # if you only wanted to read 512 bytes, do .read(512)
        in_file.close()

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'pluginupload'
        message_payload['configparams'] = compress_param(json.dumps(configparams))
        message_payload['jardata'] = encode_data(jar_data)


        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)

        # returns reply with status and pluginid
        return reply

    def update_plugin_agent(self, dst_region, dst_agent, jar_file_path):

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'controllerupdate'
        message_payload['jar_file_path'] = jar_file_path

        reply = self.messaging.global_agent_msgevent(False, message_event_type, message_payload, dst_region, dst_agent)

        # returns reply with status and pluginid
        return reply

    def get_broadcast_discovery(self, dst_region, dst_agent):

        message_event_type = 'EXEC'
        message_payload = dict()
        message_payload['action'] = 'getbroadcastdiscovery'

        # dst_region = 'global-region'
        # dst_agent = 'global-controller'

        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)
        # return reply with status code and pluginid
        return reply

    def cepadd(self, input_stream, input_stream_desc, output_stream, output_stream_desc, query, dst_region, dst_agent):


        cepparams = dict()
        cepparams['input_stream'] = input_stream
        cepparams['input_stream_desc'] = input_stream_desc
        cepparams['output_stream'] = output_stream
        cepparams['output_stream_desc'] = output_stream_desc
        cepparams['query'] = query

        message_event_type = 'CONFIG'
        message_payload = dict()
        message_payload['action'] = 'cepadd'
        message_payload['cepparams'] = compress_param(json.dumps(cepparams))

        print(cepparams)
        reply = self.messaging.global_agent_msgevent(True, message_event_type, message_payload, dst_region, dst_agent)

        print(reply)
        return reply




import time

import requests

class http_interface(object):

    def __init__(self):
        self.url = None
        self.service_key = None
        self.isconnected = False

    def api_version(self):

        version = None
        try:
            headers = {'Content-Type': 'application/json',
                       'X-Auth-API-Service-Key': self.service_key,
                       }
            version_url = self.url + '/dashboard/api/version'

            res = requests.get(url=version_url,
                               headers=headers
                               )

            response_json = res.json()
            version = response_json['version']
        except:
            print('HTTP Client: api_version failed for ' + version_url)

        return version

    def connected(self):
        return self.isconnected


    def connect(self, url, service_key):

        try:

            if url[-1] == '/':
                url = url[:-1]

            self.url = url
            self.service_key = service_key

            if self.api_version() is not None:
                self.isconnected = True

        except:
            print('HTTP Client: Failed to connect to ' + self.url)

        return self.isconnected

    #HTTP functions
    def add_repo_plugin(self, file_path, verify=False):

        # '/Users/cody/IdeaProjects/cepdemo/target/cepdemo-1.0-SNAPSHOT.jar'
        data = open(file_path, 'rb').read()
        headers = {'Content-Type': 'application/octet-stream',
                   'X-Auth-API-Service-Key': self.service_key,
                   'Content-Type': 'application/java-archive'
                   }
        upload_url = self.http_url + '/dashboard/plugins/uploadplugin'
        res = requests.post(url=upload_url,
                            data=data,
                            headers=headers,
                            )

        plugin_info = res.json()

        if verify:

            isUploaded = False

            headers = {'Content-Type': 'application/json',
                       'X-Auth-API-Service-Key': self.service_key,
                       }
            list_url = self.http_url + '/dashboard/plugins/listrepo'

            while not isUploaded:
                res = requests.get(url=list_url,
                                   headers=headers
                                   )
                plugin_list = res.json()
                print(plugin_list)
                for plugin in plugin_list['plugins']:
                    if plugin == plugin_info:
                        isUploaded = True
                time.sleep(1)

        return plugin_info



import json

class messaging(object):

    def __init__(self, ws_interface):
        self.ws_interface = ws_interface

    # WS functions
    def global_controller_msgevent(self, is_rpc, message_event_type, message_payload):
        message_info = dict()
        message_info['message_type'] = 'global_controller_msgevent'
        message_info['message_event_type'] = message_event_type
        message_info['is_rpc'] = is_rpc

        message = dict()
        message['message_info'] = message_info
        message['message_payload'] = message_payload

        #print("Sending..")
        json_message = json.dumps(message)
        #print(json_message)

        self.ws_interface.ws.send(json_message)

        if is_rpc:
            #print("Receiving...")
            json_incoming = json.loads(self.ws_interface.ws.recv())
            #print(json_incoming)
            return json_incoming
        else:
            return None

    def regional_controller_msgevent(self, is_rpc, message_event_type, message_payload):
        message_info = dict()
        message_info['message_type'] = 'regional_controller_msgevent'
        message_info['message_event_type'] = message_event_type
        message_info['is_rpc'] = is_rpc

        message = dict()
        message['message_info'] = message_info
        message['message_payload'] = message_payload

        #print("Sending..")
        json_message = json.dumps(message)
        #print(json_message)

        self.ws_interface.ws.send(json_message)

        if is_rpc:
            #print("Receiving...")
            json_incoming = json.loads(self.ws_interface.ws.recv())
            #print(json_incoming)
            return json_incoming
        else:
            return None

    def global_agent_msgevent(self, is_rpc, message_event_type, message_payload, dst_region, dst_agent):
        message_info = dict()
        message_info['message_type'] = 'global_agent_msgevent'
        message_info['message_event_type'] = message_event_type
        message_info['dst_region'] = dst_region
        message_info['dst_agent'] = dst_agent
        message_info['is_rpc'] = is_rpc

        message = dict()
        message['message_info'] = message_info
        message['message_payload'] = message_payload

        #print("Sending..")
        json_message = json.dumps(message)
        #print(json_message)

        self.ws_interface.ws.send(json_message)

        if is_rpc:
            #print("Receiving...")
            json_incoming = json.loads(self.ws_interface.ws.recv())
            #print(json_incoming)
            return json_incoming
        else:
            return None

    def regional_agent_msgevent(self, is_rpc, message_event_type, message_payload, dst_agent):
        message_info = dict()
        message_info['message_type'] = 'regional_agent_msgevent'
        message_info['message_event_type'] = message_event_type
        message_info['dst_agent'] = dst_agent
        message_info['is_rpc'] = is_rpc

        message = dict()
        message['message_info'] = message_info
        message['message_payload'] = message_payload

        #print("Sending..")
        json_message = json.dumps(message)
        #print(json_message)

        self.ws_interface.ws.send(json_message)

        if is_rpc:
            #print("Receiving...")
            json_incoming = json.loads(self.ws_interface.ws.recv())
            #print(json_incoming)
            return json_incoming
        else:
            return None

    def agent_msgevent(self, is_rpc, message_event_type, message_payload):
        message_info = dict()
        message_info['message_type'] = 'agent_msgevent'
        message_info['message_event_type'] = message_event_type
        message_info['is_rpc'] = is_rpc

        message = dict()
        message['message_info'] = message_info
        message['message_payload'] = message_payload

        #print("Sending..")
        json_message = json.dumps(message)
        #print(json_message)

        self.ws_interface.ws.send(json_message)

        if is_rpc:
            #print("Receiving...")
            json_incoming = json.loads(self.ws_interface.ws.recv())
            #print(json_incoming)
            return json_incoming
        else:
            return None

    def global_plugin_msgevent(self, is_rpc, message_event_type, message_payload, dst_region, dst_agent, dst_plugin):
        message_info = dict()
        message_info['message_type'] = 'global_plugin_msgevent'
        message_info['message_event_type'] = message_event_type
        message_info['dst_region'] = dst_region
        message_info['dst_agent'] = dst_agent
        message_info['dst_plugin'] = dst_plugin
        message_info['is_rpc'] = is_rpc

        message = dict()
        message['message_info'] = message_info
        message['message_payload'] = message_payload

        #print("Sending..")
        json_message = json.dumps(message)
        #print(json_message)

        self.ws_interface.ws.send(json_message)

        if is_rpc:
            #print("Receiving...")
            json_incoming = json.loads(self.ws_interface.ws.recv())
            #print(json_incoming)
            return json_incoming
        else:
            return None

    def regional_plugin_msgevent(self, is_rpc, message_event_type, message_payload, dst_agent, dst_plugin):
        message_info = dict()
        message_info['message_type'] = 'regional_plugin_msgevent'
        message_info['message_event_type'] = message_event_type
        message_info['dst_agent'] = dst_agent
        message_info['dst_plugin'] = dst_plugin
        message_info['is_rpc'] = is_rpc

        message = dict()
        message['message_info'] = message_info
        message['message_payload'] = message_payload

        #print("Sending..")
        json_message = json.dumps(message)
        #print(json_message)

        self.ws_interface.ws.send(json_message)

        if is_rpc:
            #print("Receiving...")
            json_incoming = json.loads(self.ws_interface.ws.recv())
            #print(json_incoming)
            return json_incoming
        else:
            return None

    def plugin_msgevent(self, is_rpc, message_event_type, message_payload, dst_plugin):
        message_info = dict()
        message_info['message_type'] = 'plugin_msgevent'
        message_info['message_event_type'] = message_event_type
        message_info['dst_plugin'] = dst_plugin
        message_info['is_rpc'] = is_rpc

        message = dict()
        message['message_info'] = message_info
        message['message_payload'] = message_payload

        #print("Sending..")
        json_message = json.dumps(message)
        #print(json_message)

        self.ws_interface.ws.send(json_message)

        if is_rpc:
            #print("Receiving...")
            json_incoming = json.loads(self.ws_interface.ws.recv())
            #print(json_incoming)
            return json_incoming
        else:
            return None
